<?php
// phpcs:disable Yoast.NamingConventions.NamespaceName
namespace Yoast\WP\SEO\Dashboard\Domain\Score_Groups\SEO_Score_Groups;

use Yoast\WP\SEO\Dashboard\Domain\Score_Groups\Score_Groups_Interface;

/**
 * This interface describes an SEO score group implementation.
 */
interface SEO_Score_Groups_Interface extends Score_Groups_Interface {}
