body.seo_page_wpseo_redirects{--tw-bg-opacity:1;background-color:rgb(241 245 249/var(--tw-bg-opacity,1));z-index:-1}body.seo_page_wpseo_redirects #wpcontent{padding-left:0!important}body.seo_page_wpseo_redirects #wpfooter{padding-right:1rem}@media (min-width:768px){body.seo_page_wpseo_redirects #wpfooter{padding-left:17rem;padding-right:2rem}}@media screen and (max-width:782px){body.seo_page_wpseo_redirects .wp-responsive-open #wpbody{right:-190px}}body.seo_page_wpseo_redirects #modal-search .yst-modal__close{margin-top:-.25rem}@media (min-width:783px) and (max-width:962px){body.seo_page_wpseo_redirects.sticky-menu .yst-root .yst-notifications--bottom-left{left:calc(160px + 2rem)}}@media (min-width:783px) and (max-width:963px){body.seo_page_wpseo_redirects.sticky-menu.auto-fold .yst-root .yst-notifications--bottom-left,body.seo_page_wpseo_redirects.sticky-menu.folded .yst-root .yst-notifications--bottom-left{left:calc(32px + 2rem)}}@media (min-width:962px){body.seo_page_wpseo_redirects.sticky-menu.folded .yst-root .yst-notifications--bottom-left{left:calc(32px + 2rem)}}@media (max-width:783px){body.seo_page_wpseo_redirects:not(.sticky-menu) .wp-responsive-open .yst-root .yst-notifications--bottom-left{left:calc(190px + 2rem)}}@media (min-width:783px){body.seo_page_wpseo_redirects .yst-root .yst-notifications--bottom-left{left:calc(160px + 2rem)}}@media (min-width:601px) and (max-width:768px){body.seo_page_wpseo_redirects .yst-root .yst-mobile-navigation__top{top:46px}}@media (min-width:783px){body.seo_page_wpseo_redirects .yst-root .yst-mobile-navigation__top{display:none}}body.seo_page_wpseo_redirects .yst-root .yst-mobile-navigation__dialog{z-index:99999}body.seo_page_wpseo_redirects .yst-root .yst-paper{margin-bottom:unset;min-height:unset;vertical-align:unset;white-space:normal}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-table-header{font-weight:500;--tw-text-opacity:1;color:rgb(30 41 59/var(--tw-text-opacity,1))}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-table-cell{padding-bottom:1.25rem;padding-top:1.25rem;--tw-text-opacity:1;color:rgb(100 116 139/var(--tw-text-opacity,1))}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-table-cell .yst-checkbox__label{margin-inline-start:2.25rem}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-checkbox__label{margin-inline-start:1.5rem}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-table-cell:first-child,body.seo_page_wpseo_redirects .yst-table--minimal table .yst-table-header:first-child{padding-inline-start:.75rem}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-dropdown-menu__item--button{justify-content:flex-start;padding:.5rem .75rem;--tw-text-opacity:1;color:rgb(71 85 105/var(--tw-text-opacity,1))}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-dropdown-menu__item--button:hover{--tw-bg-opacity:1;background-color:rgb(226 232 240/var(--tw-bg-opacity,1));--tw-text-opacity:1;color:rgb(15 23 42/var(--tw-text-opacity,1))}body.seo_page_wpseo_redirects .yst-table--minimal table .yst-dropdown-menu__list{z-index:1100!important}body.seo_page_wpseo_redirects .yst-modal__panel{max-width:32rem}body.seo_page_wpseo_redirects.rtl .yst-root .yst-replacevar .emoji-select-popover{left:0;right:auto}body.seo_page_wpseo_redirects #screen-meta-links{display:none}