.seo_page_wpseo_licenses,.yoast-seo_page_wpseo_licenses{--tw-bg-opacity:1;background-color:rgb(241 245 249/var(--tw-bg-opacity,1));z-index:-1}.seo_page_wpseo_licenses #wpcontent,.yoast-seo_page_wpseo_licenses #wpcontent{padding-left:0!important}